@@ -8,6 +8,7 @@ use Glib qw(TRUE FALSE);            # To get TRUE and FALSE
 use File::Copy;
 use Try::Tiny;
 use Data::Dumper;
+$Data::Dumper::Sortkeys = 1;
 use Config::General 2.40;
 use JSON::PP;
 use version;
@@ -16,7 +17,7 @@ BEGIN {
     use Exporter ();
     our ( $VERSION, @EXPORT_OK, %EXPORT_TAGS );
 
-    $VERSION = '1.8.10';
+    $VERSION = '1.8.11';
 
     use base qw(Exporter);
     %EXPORT_TAGS = ();    # eg: TAG => [ qw!name1 name2! ],
@@ -252,6 +253,7 @@ sub add_defaults {
         'unsharp amount'                    => 1,
         'unsharp threshold'                 => 0.05,
         'allow-batch-flatbed'               => FALSE,
+        'cancel-between-pages'              => FALSE,
         'adf-defaults-scan-all-pages'       => TRUE,
         'cycle sane handle'                 => FALSE,
         profile                             => undef,
