@@ -11,6 +11,7 @@ use Gscan2pdf::Translation '__';    # easier to extract strings with xgettext
 use Locale::gettext 1.05;           # For translations
 use feature 'switch';
 use Data::Dumper;
+$Data::Dumper::Sortkeys = 1;
 use Readonly;
 Readonly my $LAST_PAGE => -1;
 
@@ -30,9 +31,17 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::Scan::, properties => [
         FALSE,                     # default_value
         [qw/readable writable/]    # flags
     ),
+    Glib::ParamSpec->boolean(
+        'cancel-between-pages',                          # name
+        'Cancel previous page when starting new one',    # nick
+'Otherwise, some Brother scanners report out of documents, despite scanning from flatbed.'
+        ,                                                # blurb
+        FALSE,                                           # default_value
+        [qw/readable writable/]                          # flags
+    ),
 ];
 
-our $VERSION = '1.8.10';
+our $VERSION = '1.8.11';
 
 my $SANE_NAME_SCAN_TL_X   = SANE_NAME_SCAN_TL_X;
 my $SANE_NAME_SCAN_TL_Y   = SANE_NAME_SCAN_TL_Y;
@@ -361,7 +370,7 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
 # and walking the options tree, update the widgets
 
 sub set_option {
-    my ( $self, $option, $val ) = @_;
+    my ( $self, $option, $val, $uuid ) = @_;
     if ( not defined $option ) { return }
 
     if ( $option->{constraint_type} == SANE_CONSTRAINT_RANGE ) {
@@ -412,18 +421,19 @@ sub set_option {
             );
 
             # Unset the profile unless we are actively setting it
-            if ( not $self->{setting_profile} ) {
+            if ( not @{ $self->{setting_profile} } ) {
                 $self->set( 'profile', undef );
 
                 # Emit the changed-current-scan-options signal
                 # unless we are actively setting it
-                if ( not $self->{setting_current_scan_options} ) {
+                if ( not @{ $self->{setting_current_scan_options} } ) {
                     $self->signal_emit( 'changed-current-scan-options',
                         $self->get('current-scan-options'), $EMPTY );
                 }
             }
 
-            $self->signal_emit( 'changed-scan-option', $option->{name}, $val );
+            $self->signal_emit( 'changed-scan-option', $option->{name}, $val,
+                $uuid );
         },
         error_callback => sub {
             my ($message) = @_;
@@ -451,11 +461,12 @@ sub scan {
 
     my $i = 1;
     Gscan2pdf::Frontend::Image_Sane->scan_pages(
-        dir              => $self->get('dir'),
-        npages           => $npages,
-        start            => $start,
-        step             => $step,
-        started_callback => sub {
+        dir                  => $self->get('dir'),
+        npages               => $npages,
+        start                => $start,
+        step                 => $step,
+        cancel_between_pages => $self->get('cancel-between-pages'),
+        started_callback     => sub {
             if ( $npages == 0 and $self->get('max-pages') > 0 ) {
                 $npages = $self->get('max-pages');
             }
