@@ -11,6 +11,7 @@ use Thread::Queue;
 use Storable qw(freeze thaw);    # For cloning the options cache
 use Try::Tiny;
 use Data::Dumper;
+$Data::Dumper::Sortkeys = 1;
 use Glib qw(TRUE FALSE);
 use Image::Sane ':all';
 use Data::UUID;
@@ -25,7 +26,7 @@ Readonly my $MAXVAL_16_BIT  => 2**$_16_BIT - 1;
 my $uuid_object = Data::UUID->new;
 my $EMPTY       = q{};
 
-our $VERSION = '1.8.10';
+our $VERSION = '1.8.11';
 
 my ( $prog_name, $logger, %callback, $_self );
 
@@ -222,6 +223,9 @@ sub scan_page_finished_callback {
         }
         return;
     }
+    elsif ( $options{cancel_between_pages} ) {
+        _enqueue_request( 'cancel', { uuid => $uuid_object->create_str } );
+    }
 
     if ( not defined $options{step} ) { $options{step} = 1 }
     $options{start} += $options{step};
@@ -558,8 +562,8 @@ sub _thread_set_option {
               . ( $opt->{type} == SANE_TYPE_BUTTON ? $EMPTY : " to $value" )
               . " returned status $status ("
               . Image::Sane::strstatus($status)
-              . ") with info "
-              . ( defined $info ? $info : "undefined" ) );
+              . ') with info '
+              . ( defined $info ? $info : 'undefined' ) );
     }
 
     # $info could be undefined if status is invalid
